from django.contrib.auth import logout
from django.contrib.auth.mixins import LoginRequiredMixin
import shopify

from .helpers import add_query_parameters_to_url


class ShopifyLoginRequiredMixin(LoginRequiredMixin):
    """
    Mixin which verifies that the current shop is authenticated and app is installed in shopify.
    """
    
    def handle_no_permission(self):
        shopify_params = {
            k: self.request.GET[k]
            for k in ["shop", "timestamp", "signature", "hmac"]
            if k in self.request.GET
        }

        # Add the Shopify authentication parameters to the login URL.
        self.login_url = add_query_parameters_to_url(
            self.get_login_url(), shopify_params
        )
        return super().handle_no_permission()


    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_authenticated:
            return self.handle_no_permission()

        # Handle mismatch between request and session
        shop_name = request.GET.get("shop", None)
        if shop_name and request.user.myshopify_domain != shop_name:
            logout(request)
            return self.handle_no_permission()

        # Initialize shop making sure we can still access it
        try:
            with request.user.session:
                self.shop = shopify.Shop.current()
        except:
            logout(request)
            return self.handle_no_permission()
        return super().dispatch(request, *args, **kwargs)



import logging
from ua_parser import user_agent_parser


def is_mac_os_10_14(os):
    return "Mac OS X" in os["family"] and os["major"] == "10" and os["minor"] == "14"


def is_ios_12(os):
    return "iOS" in os["family"] and os["major"] == "12"


def is_safari(browser):
    return browser["family"] == "Safari"


def should_set_none(request):
    user_agent = request.META.get("HTTP_USER_AGENT")
    if not user_agent:
        return True

    try:
        parsed_string = user_agent_parser.Parse(user_agent)
        browser = parsed_string["user_agent"]
        os = parsed_string["os"]

        if "Chrom" in browser["family"] and int(browser["major"]) < 67:
            return False
        if is_ios_12(os):
            return False
        if is_mac_os_10_14(os) and is_safari(browser):
            return False
    except:
        logging.exception("User Agent resolution failed")

    return True


class SamesiteCookieMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        if should_set_none(request):
            for cookie in response.cookies.keys():
                if cookie in ["sessionid", "csrftoken", "a_third_party_cookie"]:
                    response.cookies[cookie]["samesite"] = "None"

        return response



from django.conf.urls import url

from . import views

urlpatterns = [
  url(r'^finalize/$',     views.finalize),
  url(r'^authenticate/$', views.authenticate),
  url(r'^$',              views.login),
  url(r'^check-cookie$',  views.check_cookie),
]
